@@ -12,14 +12,8 @@ Or in file browser, right click on file.dcm, open with custom command:
 python3 dicomtree.py
 """
 
-from __future__ import print_function
 
-from pydicom import compat
-
-if compat.in_py2:
-    import Tix as tkinter_tix
-else:
-    import tkinter.tix as tkinter_tix
+import tkinter.tix as tkinter_tix
 
 print(__doc__)
 
@@ -64,8 +58,8 @@ def recurse_tree(tree, dataset, parent, hide=False):
     # order the dicom tags
     for data_element in dataset:
         node_id = parent + "." + hex(id(data_element))
-        if isinstance(data_element.value, compat.text_type):
-            tree.hlist.add(node_id, text=compat.text_type(data_element))
+        if isinstance(data_element.value, str):
+            tree.hlist.add(node_id, text=str(data_element))
         else:
             tree.hlist.add(node_id, text=str(data_element))
         if hide:
