@@ -10,13 +10,9 @@ Very simple app to display unicode person names.
 # authors : Guillaume Lemaitre <g.lemaitre58@gmail.com>
 # license : MIT
 
-from pydicom import compat
 from pydicom.valuerep import PersonNameUnicode
 
-if compat.in_py2:
-    import Tkinter as tkinter
-else:
-    import tkinter
+import tkinter
 
 print(__doc__)
 
