@@ -9,7 +9,7 @@ from struct import (Struct, unpack)
 import sys
 from typing import (
     BinaryIO, Union, Optional, List, Any, Callable, cast, MutableSequence,
-    Type, Iterator, Dict
+    Iterator, Dict
 )
 import warnings
 import zlib
@@ -24,6 +24,7 @@ from pydicom.dataelem import (
 from pydicom.dataset import Dataset, FileDataset, FileMetaDataset
 from pydicom.dicomdir import DicomDir
 from pydicom.errors import InvalidDicomError
+from pydicom.filebase import DicomFileLike
 from pydicom.fileutil import (
     read_undefined_length_value, path_from_pathlike, PathType, _unpack_tag
 )
@@ -732,7 +733,8 @@ def read_preamble(fp: BinaryIO, force: bool) -> Optional[bytes]:
 
 
 def _at_pixel_data(tag: BaseTag, VR: Optional[str], length: int) -> bool:
-    return cast(bool, tag == 0x7fe00010)
+    pixel_data_tags = {0x7fe00010, 0x7fe00009, 0x7fe00008}
+    return tag in pixel_data_tags
 
 
 def read_partial(
@@ -901,7 +903,7 @@ def read_partial(
 
 
 def dcmread(
-    fp: Union[PathType, BinaryIO],
+    fp: Union[PathType, BinaryIO, DicomFileLike],
     defer_size: Optional[Union[str, int, float]] = None,
     stop_before_pixels: bool = False,
     force: bool = False,
