@@ -43,22 +43,28 @@ table below.
 | (0028,0100) | BitsAllocated             | 1    | 1, 8, 16, 32, | Required |
 |             |                           |      | 64            |          |
 +-------------+---------------------------+------+---------------+----------+
-| (0028,0103) | PixelRepresentation       | 1    | 0, 1          | Required |
+| (0028,0101) | BitsStored                | 1    | 1, 8, 12, 16  | Optional |
++-------------+---------------------------+------+---------------+----------+
+| (0028,0103) | PixelRepresentation       | 1C   | 0, 1          | Optional |
 +-------------+---------------------------+------+---------------+----------+
 
 """
 
+from typing import TYPE_CHECKING, cast
+import warnings
+
 try:
     import numpy as np
     HAVE_NP = True
 except ImportError:
     HAVE_NP = False
 
-import warnings
-
 from pydicom.pixel_data_handlers.util import pixel_dtype, get_expected_length
 import pydicom.uid
 
+if TYPE_CHECKING:  # pragma: no cover
+    from pydicom.dataset import Dataset
+
 HANDLER_NAME = 'Numpy'
 
 DEPENDENCIES = {
@@ -73,12 +79,12 @@ SUPPORTED_TRANSFER_SYNTAXES = [
 ]
 
 
-def is_available():
+def is_available() -> bool:
     """Return ``True`` if the handler has its dependencies met."""
     return HAVE_NP
 
 
-def supports_transfer_syntax(transfer_syntax):
+def supports_transfer_syntax(transfer_syntax: pydicom.uid.UID) -> bool:
     """Return ``True`` if the handler supports the `transfer_syntax`.
 
     Parameters
@@ -90,7 +96,7 @@ def supports_transfer_syntax(transfer_syntax):
     return transfer_syntax in SUPPORTED_TRANSFER_SYNTAXES
 
 
-def needs_to_convert_to_RGB(ds):
+def needs_to_convert_to_RGB(ds: "Dataset") -> bool:
     """Return ``True`` if the *Pixel Data* should to be converted from YCbCr to
     RGB.
 
@@ -99,7 +105,7 @@ def needs_to_convert_to_RGB(ds):
     return False
 
 
-def should_change_PhotometricInterpretation_to_RGB(ds):
+def should_change_PhotometricInterpretation_to_RGB(ds: "Dataset") -> bool:
     """Return ``True`` if the *Photometric Interpretation* should be changed
     to RGB.
 
@@ -108,13 +114,18 @@ def should_change_PhotometricInterpretation_to_RGB(ds):
     return False
 
 
-def pack_bits(arr):
+def pack_bits(arr: "np.ndarray", pad: bool = True) -> bytes:
     """Pack a binary :class:`numpy.ndarray` for use with *Pixel Data*.
 
     .. versionadded:: 1.2
 
     Should be used in conjunction with (0028,0100) *Bits Allocated* = 1.
 
+    .. versionchanged:: 2.1
+
+        Added the `pad` keyword parameter and changed to allow `arr` to be
+        2 or 3D.
+
     Parameters
     ----------
     arr : numpy.ndarray
@@ -122,7 +133,13 @@ def pack_bits(arr):
         only contain integer values of 0 and 1 and must have an 'uint'  or
         'int' :class:`numpy.dtype`. For the sake of efficiency it's recommended
         that the length of `arr` be a multiple of 8 (i.e. that any empty
-        bit-padding to round out the byte has already been added).
+        bit-padding to round out the byte has already been added). The input
+        `arr` should either be shaped as (rows, columns) or (frames, rows,
+        columns) or the equivalent 1D array used to ensure that the packed
+        data is in the correct order.
+    pad : bool, optional
+        If ``True`` (default) then add a null byte to the end of the packed
+        data to ensure even length, otherwise no padding will be added.
 
     Returns
     -------
@@ -150,7 +167,7 @@ def pack_bits(arr):
         )
 
     if len(arr.shape) > 1:
-        raise ValueError("Only 1D arrays are supported.")
+        arr = arr.ravel()
 
     # The array length must be a multiple of 8, pad the end
     if arr.shape[0] % 8:
@@ -161,10 +178,14 @@ def pack_bits(arr):
     arr = np.fliplr(arr)
     arr = np.packbits(arr.astype('uint8'))
 
-    return arr.tobytes()
+    packed: bytes = arr.tobytes()
+    if pad:
+        return packed + b'\x00' if len(packed) % 2 else packed
+
+    return packed
 
 
-def unpack_bits(bytestream):
+def unpack_bits(bytestream: bytes) -> "np.ndarray":
     """Unpack bit packed *Pixel Data* or *Overlay Data* into a
     :class:`numpy.ndarray`.
 
@@ -201,10 +222,10 @@ def unpack_bits(bytestream):
     # -> [0 0 0 0 0 0 1 1 1 0 0 1 0 0 0 0]
     arr = np.ravel(arr)
 
-    return arr
+    return cast("np.ndarray", arr)
 
 
-def get_pixeldata(ds, read_only=False):
+def get_pixeldata(ds: "Dataset", read_only: bool = False) -> "np.ndarray":
     """Return a :class:`numpy.ndarray` of the pixel data.
 
     .. versionchanged:: 1.4
@@ -265,16 +286,28 @@ def get_pixeldata(ds, read_only=False):
             "the dataset"
         )
 
+    # Attributes required by both Floating Point Image Pixel Module Attributes
+    # and Image Pixel Description Macro Attributes
     required_elements = [
-        'BitsAllocated', 'Rows', 'Columns', 'PixelRepresentation',
+        'BitsAllocated', 'Rows', 'Columns',
         'SamplesPerPixel', 'PhotometricInterpretation'
     ]
+    if px_keyword[0] == 'PixelData':
+        # Attributess required by Image Pixel Description Macro Attributes
+        required_elements.extend(['PixelRepresentation', 'BitsStored'])
     missing = [elem for elem in required_elements if elem not in ds]
     if missing:
         raise AttributeError(
             "Unable to convert the pixel data as the following required "
             "elements are missing from the dataset: " + ", ".join(missing)
         )
+    if ds.SamplesPerPixel > 1:
+        if not hasattr(ds, 'PlanarConfiguration'):
+            raise AttributeError(
+                "Unable to convert the pixel data as the following "
+                "conditionally required element is missing from the dataset: "
+                "PlanarConfiguration"
+            )
 
     # May be Pixel Data, Float Pixel Data or Double Float Pixel Data
     pixel_data = getattr(ds, px_keyword[0])
@@ -348,6 +381,6 @@ def get_pixeldata(ds, read_only=False):
         ds.PhotometricInterpretation = "RGB"
 
     if not read_only and ds.BitsAllocated > 1:
-        return arr.copy()
+        return cast("np.ndarray", arr.copy())
 
-    return arr
+    return cast("np.ndarray", arr)
