@@ -3,6 +3,9 @@
 
 from struct import unpack
 from sys import byteorder
+from typing import (
+    Dict, Optional, Union, List, Tuple, TYPE_CHECKING, cast, Iterable
+)
 import warnings
 
 try:
@@ -14,8 +17,15 @@ except ImportError:
 from pydicom.data import get_palette_files
 from pydicom.uid import UID
 
+if TYPE_CHECKING:  # pragma: no cover
+    from pydicom.dataset import Dataset, FileMetaDataset, FileDataset
 
-def apply_color_lut(arr, ds=None, palette=None):
+
+def apply_color_lut(
+    arr: "np.ndarray",
+    ds: Optional["Dataset"] = None,
+    palette: Optional[Union[str, UID]] = None
+) -> "np.ndarray":
     """Apply a color palette lookup table to `arr`.
 
     .. versionadded:: 1.4
@@ -104,6 +114,8 @@ def apply_color_lut(arr, ds=None, palette=None):
         except KeyError:
             raise ValueError("Unknown palette '{}'".format(palette))
 
+    ds = cast("Dataset", ds)
+
     # C.8.16.2.1.1.1: Supplemental Palette Color LUT
     # TODO: Requires greyscale visualisation pipeline
     if getattr(ds, 'PixelPresentation', None) in ['MIXED', 'COLOR']:
@@ -116,7 +128,7 @@ def apply_color_lut(arr, ds=None, palette=None):
         raise ValueError("No suitable Palette Color Lookup Table Module found")
 
     # All channels are supposed to be identical
-    lut_desc = ds.RedPaletteColorLookupTableDescriptor
+    lut_desc = cast(List[int], ds.RedPaletteColorLookupTableDescriptor)
     # A value of 0 = 2^16 entries
     nr_entries = lut_desc[0] or 2**16
 
@@ -129,22 +141,28 @@ def apply_color_lut(arr, ds=None, palette=None):
     luts = []
     if 'RedPaletteColorLookupTableData' in ds:
         # LUT Data is described by PS3.3, C.7.6.3.1.6
-        r_lut = ds.RedPaletteColorLookupTableData
-        g_lut = ds.GreenPaletteColorLookupTableData
-        b_lut = ds.BluePaletteColorLookupTableData
-        a_lut = getattr(ds, 'AlphaPaletteColorLookupTableData', None)
+        r_lut = cast(bytes, ds.RedPaletteColorLookupTableData)
+        g_lut = cast(bytes, ds.GreenPaletteColorLookupTableData)
+        b_lut = cast(bytes, ds.BluePaletteColorLookupTableData)
+        a_lut = cast(
+            Optional[bytes],
+            getattr(ds, 'AlphaPaletteColorLookupTableData', None)
+        )
 
         actual_depth = len(r_lut) / nr_entries * 8
         dtype = np.dtype('uint{:.0f}'.format(actual_depth))
 
-        for lut in [ii for ii in [r_lut, g_lut, b_lut, a_lut] if ii]:
-            luts.append(np.frombuffer(lut, dtype=dtype))
+        for lut_bytes in [ii for ii in [r_lut, g_lut, b_lut, a_lut] if ii]:
+            luts.append(np.frombuffer(lut_bytes, dtype=dtype))
     elif 'SegmentedRedPaletteColorLookupTableData' in ds:
         # Segmented LUT Data is described by PS3.3, C.7.9.2
-        r_lut = ds.SegmentedRedPaletteColorLookupTableData
-        g_lut = ds.SegmentedGreenPaletteColorLookupTableData
-        b_lut = ds.SegmentedBluePaletteColorLookupTableData
-        a_lut = getattr(ds, 'SegmentedAlphaPaletteColorLookupTableData', None)
+        r_lut = cast(bytes, ds.SegmentedRedPaletteColorLookupTableData)
+        g_lut = cast(bytes, ds.SegmentedGreenPaletteColorLookupTableData)
+        b_lut = cast(bytes, ds.SegmentedBluePaletteColorLookupTableData)
+        a_lut = cast(
+            Optional[bytes],
+            getattr(ds, 'SegmentedAlphaPaletteColorLookupTableData', None)
+        )
 
         endianness = '<' if ds.is_little_endian else '>'
         byte_depth = nominal_depth // 8
@@ -154,15 +172,15 @@ def apply_color_lut(arr, ds=None, palette=None):
         for seg in [ii for ii in [r_lut, g_lut, b_lut, a_lut] if ii]:
             len_seg = len(seg) // byte_depth
             s_fmt = endianness + str(len_seg) + fmt
-            lut = _expand_segmented_lut(unpack(s_fmt, seg), s_fmt)
-            luts.append(np.asarray(lut, dtype=dtype))
+            lut_ints = _expand_segmented_lut(unpack(s_fmt, seg), s_fmt)
+            luts.append(np.asarray(lut_ints, dtype=dtype))
     else:
         raise ValueError("No suitable Palette Color Lookup Table Module found")
 
     if actual_depth not in [8, 16]:
         raise ValueError(
-            "The bit depth of the LUT data '{:.1f}' is invalid (only 8 or 16 "
-            "bits per entry allowed)".format(actual_depth)
+            f"The bit depth of the LUT data '{actual_depth:.1f}' "
+            "is invalid (only 8 or 16 bits per entry allowed)"
         )
 
     lut_lengths = [len(ii) for ii in luts]
@@ -186,7 +204,7 @@ def apply_color_lut(arr, ds=None, palette=None):
     return out
 
 
-def apply_modality_lut(arr, ds):
+def apply_modality_lut(arr: "np.ndarray", ds: "Dataset") -> "np.ndarray":
     """Apply a modality lookup table or rescale operation to `arr`.
 
     .. versionadded:: 1.4
@@ -227,21 +245,23 @@ def apply_modality_lut(arr, ds):
       <part04/sect_N.2.html#sect_N.2.1.1>`
     """
     if 'ModalityLUTSequence' in ds:
-        item = ds.ModalityLUTSequence[0]
-        nr_entries = item.LUTDescriptor[0] or 2**16
-        first_map = item.LUTDescriptor[1]
-        nominal_depth = item.LUTDescriptor[2]
+        item = cast(List["Dataset"], ds.ModalityLUTSequence)[0]
+        nr_entries = cast(List[int], item.LUTDescriptor)[0] or 2**16
+        first_map = cast(List[int], item.LUTDescriptor)[1]
+        nominal_depth = cast(List[int], item.LUTDescriptor)[2]
 
         dtype = 'uint{}'.format(nominal_depth)
 
         # Ambiguous VR, US or OW
+        unc_data: Iterable[int]
         if item['LUTData'].VR == 'OW':
             endianness = '<' if ds.is_little_endian else '>'
             unpack_fmt = '{}{}H'.format(endianness, nr_entries)
-            lut_data = unpack(unpack_fmt, item.LUTData)
+            unc_data = unpack(unpack_fmt, cast(bytes, item.LUTData))
         else:
-            lut_data = item.LUTData
-        lut_data = np.asarray(lut_data, dtype=dtype)
+            unc_data = cast(List[int], item.LUTData)
+
+        lut_data: "np.ndarray" = np.asarray(unc_data, dtype=dtype)
 
         # IVs < `first_map` get set to first LUT entry (i.e. index 0)
         clipped_iv = np.zeros(arr.shape, dtype=arr.dtype)
@@ -252,19 +272,28 @@ def apply_modality_lut(arr, ds):
         # IVs > number of entries get set to last entry
         np.clip(clipped_iv, 0, nr_entries - 1, out=clipped_iv)
 
-        return lut_data[clipped_iv]
+        return cast("np.ndarray", lut_data[clipped_iv])
     elif 'RescaleSlope' in ds and 'RescaleIntercept' in ds:
-        arr = arr.astype(np.float64) * ds.RescaleSlope
-        arr += ds.RescaleIntercept
+        arr = arr.astype(np.float64) * cast(float, ds.RescaleSlope)
+        arr += cast(float, ds.RescaleIntercept)
 
     return arr
 
 
-def apply_voi_lut(arr, ds, index=0):
+def apply_voi_lut(
+    arr: "np.ndarray",
+    ds: "Dataset",
+    index: int = 0,
+    prefer_lut: bool = True
+) -> "np.ndarray":
     """Apply a VOI lookup table or windowing operation to `arr`.
 
     .. versionadded:: 1.4
 
+    .. versionchanged:: 2.1
+
+        Added the `prefer_lut` keyword parameter
+
     Parameters
     ----------
     arr : numpy.ndarray
@@ -279,8 +308,12 @@ def apply_voi_lut(arr, ds, index=0):
         ``np.float64``. If neither are present then `arr` will be returned
         unchanged.
     index : int, optional
-        Where the VOI LUT Module contains multiple possible views, this is
+        When the VOI LUT Module contains multiple alternative views, this is
         the index of the view to return (default ``0``).
+    prefer_lut : bool
+        When the VOI LUT Module contains both *Window Width*/*Window Center*
+        and *VOI LUT Sequence*, if ``True`` (default) then apply the VOI LUT,
+        otherwise apply the windowing operation.
 
     Returns
     -------
@@ -296,6 +329,8 @@ def apply_voi_lut(arr, ds, index=0):
     See Also
     --------
     :func:`~pydicom.pixel_data_handlers.util.apply_modality_lut`
+    :func:`~pydicom.pixel_data_handlers.util.apply_voi`
+    :func:`~pydicom.pixel_data_handlers.util.apply_windowing`
 
     References
     ----------
@@ -306,134 +341,277 @@ def apply_voi_lut(arr, ds, index=0):
     * DICOM Standard, Part 4, :dcm:`Annex N.2.1.1
       <part04/sect_N.2.html#sect_N.2.1.1>`
     """
+    valid_voi = False
     if 'VOILUTSequence' in ds:
-        # VOI LUT Sequence contains one or more items
-        item = ds.VOILUTSequence[index]
-        nr_entries = item.LUTDescriptor[0] or 2**16
-        first_map = item.LUTDescriptor[1]
+        ds.VOILUTSequence = cast(List["Dataset"], ds.VOILUTSequence)
+        valid_voi = None not in [
+            ds.VOILUTSequence[0].get('LUTDescriptor', None),
+            ds.VOILUTSequence[0].get('LUTData', None)
+        ]
+    valid_windowing = None not in [
+        ds.get('WindowCenter', None),
+        ds.get('WindowWidth', None)
+    ]
 
-        # PS3.3 C.8.11.3.1.5: may be 8, 10-16
-        nominal_depth = item.LUTDescriptor[2]
-        if nominal_depth in list(range(10, 17)):
-            dtype = 'uint16'
-        elif nominal_depth == 8:
-            dtype = 'uint8'
-        else:
-            raise NotImplementedError(
-                "'{}' bits per LUT entry is not supported"
-                .format(nominal_depth)
-            )
+    if valid_voi and valid_windowing:
+        if prefer_lut:
+            return apply_voi(arr, ds, index)
 
-        # Ambiguous VR, US or OW
-        if item['LUTData'].VR == 'OW':
-            endianness = '<' if ds.is_little_endian else '>'
-            unpack_fmt = '{}{}H'.format(endianness, nr_entries)
-            lut_data = unpack(unpack_fmt, item.LUTData)
-        else:
-            lut_data = item.LUTData
-        lut_data = np.asarray(lut_data, dtype=dtype)
+        return apply_windowing(arr, ds, index)
 
-        # IVs < `first_map` get set to first LUT entry (i.e. index 0)
-        clipped_iv = np.zeros(arr.shape, dtype=arr.dtype)
-        # IVs >= `first_map` are mapped by the VOI LUT
-        # `first_map` may be negative, positive or 0
-        mapped_pixels = arr >= first_map
-        clipped_iv[mapped_pixels] = arr[mapped_pixels] - first_map
-        # IVs > number of entries get set to last entry
-        np.clip(clipped_iv, 0, nr_entries - 1, out=clipped_iv)
+    if valid_voi:
+        return apply_voi(arr, ds, index)
 
-        return lut_data[clipped_iv]
-    elif 'WindowCenter' in ds and 'WindowWidth' in ds:
-        if ds.PhotometricInterpretation not in ['MONOCHROME1', 'MONOCHROME2']:
-            raise ValueError(
-                "When performing a windowing operation only 'MONOCHROME1' and "
-                "'MONOCHROME2' are allowed for (0028,0004) Photometric "
-                "Interpretation"
-            )
-
-        # May be LINEAR (default), LINEAR_EXACT, SIGMOID or not present, VM 1
-        voi_func = getattr(ds, 'VOILUTFunction', 'LINEAR').upper()
-        # VR DS, VM 1-n
-        elem = ds['WindowCenter']
-        center = elem.value[index] if elem.VM > 1 else elem.value
-        elem = ds['WindowWidth']
-        width = elem.value[index] if elem.VM > 1 else elem.value
-
-        # The output range depends on whether or not a modality LUT or rescale
-        #   operation has been applied
-        if 'ModalityLUTSequence' in ds:
-            # Unsigned - see PS3.3 C.11.1.1.1
-            y_min = 0
-            bit_depth = ds.ModalityLUTSequence[0].LUTDescriptor[2]
-            y_max = 2**bit_depth - 1
-        elif ds.PixelRepresentation == 0:
-            # Unsigned
-            y_min = 0
-            y_max = 2**ds.BitsStored - 1
-        else:
-            # Signed
-            y_min = -2**(ds.BitsStored - 1)
-            y_max = 2**(ds.BitsStored - 1) - 1
-
-        if 'RescaleSlope' in ds and 'RescaleIntercept' in ds:
-            # Otherwise its the actual data range
-            y_min = y_min * ds.RescaleSlope + ds.RescaleIntercept
-            y_max = y_max * ds.RescaleSlope + ds.RescaleIntercept
-
-        y_range = y_max - y_min
-        arr = arr.astype('float64')
-
-        if voi_func in ['LINEAR', 'LINEAR_EXACT']:
-            # PS3.3 C.11.2.1.2.1 and C.11.2.1.3.2
-            if voi_func == 'LINEAR':
-                if width < 1:
-                    raise ValueError(
-                        "The (0028,1051) Window Width must be greater than or "
-                        "equal to 1 for a 'LINEAR' windowing operation"
-                    )
-                center -= 0.5
-                width -= 1
-            elif width <= 0:
-                raise ValueError(
-                    "The (0028,1051) Window Width must be greater than 0 "
-                    "for a 'LINEAR_EXACT' windowing operation"
-                )
-
-            below = arr <= (center - width / 2)
-            above = arr > (center + width / 2)
-            between = np.logical_and(~below, ~above)
-
-            arr[below] = y_min
-            arr[above] = y_max
-            if between.any():
-                arr[between] = (
-                    ((arr[between] - center) / width + 0.5) * y_range + y_min
-                )
-        elif voi_func == 'SIGMOID':
-            # PS3.3 C.11.2.1.3.1
-            if width <= 0:
-                raise ValueError(
-                    "The (0028,1051) Window Width must be greater than 0 "
-                    "for a 'SIGMOID' windowing operation"
-                )
-
-            arr = y_range / (1 + np.exp(-4 * (arr - center) / width)) + y_min
-        else:
-            raise ValueError(
-                "Unsupported (0028,1056) VOI LUT Function value '{}'"
-                .format(voi_func)
-            )
+    if valid_windowing:
+        return apply_windowing(arr, ds, index)
 
     return arr
 
 
-def convert_color_space(arr, current, desired):
+def apply_voi(
+    arr: "np.ndarray", ds: "Dataset", index: int = 0
+) -> "np.ndarray":
+    """Apply a VOI lookup table to `arr`.
+
+    .. versionadded:: 2.1
+
+    Parameters
+    ----------
+    arr : numpy.ndarray
+        The :class:`~numpy.ndarray` to apply the VOI LUT to.
+    ds : dataset.Dataset
+        A dataset containing a :dcm:`VOI LUT Module<part03/sect_C.11.2.html>`.
+        If (0028,3010) *VOI LUT Sequence* is present then returns an array
+        of ``np.uint8`` or ``np.uint16``, depending on the 3rd value of
+        (0028,3002) *LUT Descriptor*, otherwise `arr` will be returned
+        unchanged.
+    index : int, optional
+        When the VOI LUT Module contains multiple alternative views, this is
+        the index of the view to return (default ``0``).
+
+    Returns
+    -------
+    numpy.ndarray
+        An array with applied VOI LUT.
+
+    See Also
+    --------
+    :func:`~pydicom.pixel_data_handlers.util.apply_modality_lut`
+    :func:`~pydicom.pixel_data_handlers.util.apply_windowing`
+
+    References
+    ----------
+    * DICOM Standard, Part 3, :dcm:`Annex C.11.2
+      <part03/sect_C.11.html#sect_C.11.2>`
+    * DICOM Standard, Part 3, :dcm:`Annex C.8.11.3.1.5
+      <part03/sect_C.8.11.3.html#sect_C.8.11.3.1.5>`
+    * DICOM Standard, Part 4, :dcm:`Annex N.2.1.1
+      <part04/sect_N.2.html#sect_N.2.1.1>`
+    """
+    if "VOILUTSequence" not in ds:
+        return arr
+
+    if not np.issubdtype(arr.dtype, np.integer):
+        warnings.warn(
+            "Applying a VOI LUT on a float input array may give "
+            "incorrect results"
+        )
+
+    # VOI LUT Sequence contains one or more items
+    item = cast(List["Dataset"], ds.VOILUTSequence)[index]
+    lut_descriptor = cast(List[int], item.LUTDescriptor)
+    nr_entries = lut_descriptor[0] or 2**16
+    first_map = lut_descriptor[1]
+
+    # PS3.3 C.8.11.3.1.5: may be 8, 10-16
+    nominal_depth = lut_descriptor[2]
+    if nominal_depth in list(range(10, 17)):
+        dtype = 'uint16'
+    elif nominal_depth == 8:
+        dtype = 'uint8'
+    else:
+        raise NotImplementedError(
+            f"'{nominal_depth}' bits per LUT entry is not supported"
+        )
+
+    # Ambiguous VR, US or OW
+    unc_data: Iterable[int]
+    if item['LUTData'].VR == 'OW':
+        endianness = '<' if ds.is_little_endian else '>'
+        unpack_fmt = f'{endianness}{nr_entries}H'
+        unc_data = unpack(unpack_fmt, cast(bytes, item.LUTData))
+    else:
+        unc_data = cast(List[int], item.LUTData)
+
+    lut_data: "np.ndarray" = np.asarray(unc_data, dtype=dtype)
+
+    # IVs < `first_map` get set to first LUT entry (i.e. index 0)
+    clipped_iv = np.zeros(arr.shape, dtype=dtype)
+    # IVs >= `first_map` are mapped by the VOI LUT
+    # `first_map` may be negative, positive or 0
+    mapped_pixels = arr >= first_map
+    clipped_iv[mapped_pixels] = arr[mapped_pixels] - first_map
+    # IVs > number of entries get set to last entry
+    np.clip(clipped_iv, 0, nr_entries - 1, out=clipped_iv)
+
+    return cast("np.ndarray", lut_data[clipped_iv])
+
+
+def apply_windowing(
+    arr: "np.ndarray", ds: "Dataset", index: int = 0
+) -> "np.ndarray":
+    """Apply a windowing operation to `arr`.
+
+    .. versionadded:: 2.1
+
+    Parameters
+    ----------
+    arr : numpy.ndarray
+        The :class:`~numpy.ndarray` to apply the windowing operation to.
+    ds : dataset.Dataset
+        A dataset containing a :dcm:`VOI LUT Module<part03/sect_C.11.2.html>`.
+        If (0028,1050) *Window Center* and (0028,1051) *Window Width* are
+        present then returns an array of ``np.float64``, otherwise `arr` will
+        be returned unchanged.
+    index : int, optional
+        When the VOI LUT Module contains multiple alternative views, this is
+        the index of the view to return (default ``0``).
+
+    Returns
+    -------
+    numpy.ndarray
+        An array with applied windowing operation.
+
+    Notes
+    -----
+    When the dataset requires a modality LUT or rescale operation as part of
+    the Modality LUT module then that must be applied before any windowing
+    operation.
+
+    See Also
+    --------
+    :func:`~pydicom.pixel_data_handlers.util.apply_modality_lut`
+    :func:`~pydicom.pixel_data_handlers.util.apply_voi`
+
+    References
+    ----------
+    * DICOM Standard, Part 3, :dcm:`Annex C.11.2
+      <part03/sect_C.11.html#sect_C.11.2>`
+    * DICOM Standard, Part 3, :dcm:`Annex C.8.11.3.1.5
+      <part03/sect_C.8.11.3.html#sect_C.8.11.3.1.5>`
+    * DICOM Standard, Part 4, :dcm:`Annex N.2.1.1
+      <part04/sect_N.2.html#sect_N.2.1.1>`
+    """
+    if "WindowWidth" not in ds and "WindowCenter" not in ds:
+        return arr
+
+    if ds.PhotometricInterpretation not in ['MONOCHROME1', 'MONOCHROME2']:
+        raise ValueError(
+            "When performing a windowing operation only 'MONOCHROME1' and "
+            "'MONOCHROME2' are allowed for (0028,0004) Photometric "
+            "Interpretation"
+        )
+
+    # May be LINEAR (default), LINEAR_EXACT, SIGMOID or not present, VM 1
+    voi_func = cast(str, getattr(ds, 'VOILUTFunction', 'LINEAR')).upper()
+    # VR DS, VM 1-n
+    elem = ds['WindowCenter']
+    center = (
+        cast(List[float], elem.value)[index] if elem.VM > 1 else elem.value
+    )
+    center = cast(float, center)
+    elem = ds['WindowWidth']
+    width = cast(List[float], elem.value)[index] if elem.VM > 1 else elem.value
+    width = cast(float, width)
+
+    # The output range depends on whether or not a modality LUT or rescale
+    #   operation has been applied
+    ds.BitsStored = cast(int, ds.BitsStored)
+    y_min: float
+    y_max: float
+    if 'ModalityLUTSequence' in ds:
+        # Unsigned - see PS3.3 C.11.1.1.1
+        y_min = 0
+        item = cast(List["Dataset"], ds.ModalityLUTSequence)[0]
+        bit_depth = cast(List[int], item.LUTDescriptor)[2]
+        y_max = 2**bit_depth - 1
+    elif ds.PixelRepresentation == 0:
+        # Unsigned
+        y_min = 0
+        y_max = 2**ds.BitsStored - 1
+    else:
+        # Signed
+        y_min = -2**(ds.BitsStored - 1)
+        y_max = 2**(ds.BitsStored - 1) - 1
+
+    slope = ds.get('RescaleSlope', None)
+    intercept = ds.get('RescaleIntercept', None)
+    if slope is not None and intercept is not None:
+        ds.RescaleSlope = cast(float, ds.RescaleSlope)
+        ds.RescaleIntercept = cast(float, ds.RescaleIntercept)
+        # Otherwise its the actual data range
+        y_min = y_min * ds.RescaleSlope + ds.RescaleIntercept
+        y_max = y_max * ds.RescaleSlope + ds.RescaleIntercept
+
+    y_range = y_max - y_min
+    arr = arr.astype('float64')
+
+    if voi_func in ['LINEAR', 'LINEAR_EXACT']:
+        # PS3.3 C.11.2.1.2.1 and C.11.2.1.3.2
+        if voi_func == 'LINEAR':
+            if width < 1:
+                raise ValueError(
+                    "The (0028,1051) Window Width must be greater than or "
+                    "equal to 1 for a 'LINEAR' windowing operation"
+                )
+            center -= 0.5
+            width -= 1
+        elif width <= 0:
+            raise ValueError(
+                "The (0028,1051) Window Width must be greater than 0 "
+                "for a 'LINEAR_EXACT' windowing operation"
+            )
+
+        below = arr <= (center - width / 2)
+        above = arr > (center + width / 2)
+        between = np.logical_and(~below, ~above)
+
+        arr[below] = y_min
+        arr[above] = y_max
+        if between.any():
+            arr[between] = (
+                ((arr[between] - center) / width + 0.5) * y_range + y_min
+            )
+    elif voi_func == 'SIGMOID':
+        # PS3.3 C.11.2.1.3.1
+        if width <= 0:
+            raise ValueError(
+                "The (0028,1051) Window Width must be greater than 0 "
+                "for a 'SIGMOID' windowing operation"
+            )
+
+        arr = y_range / (1 + np.exp(-4 * (arr - center) / width)) + y_min
+    else:
+        raise ValueError(
+            f"Unsupported (0028,1056) VOI LUT Function value '{voi_func}'"
+        )
+
+    return arr
+
+
+def convert_color_space(
+    arr: "np.ndarray", current: str, desired: str, per_frame: bool = False
+) -> "np.ndarray":
     """Convert the image(s) in `arr` from one color space to another.
 
     .. versionchanged:: 1.4
 
         Added support for ``YBR_FULL_422``
 
+    .. versionchanged:: 2.2
+
+        Added `per_frame` keyword parameter.
+
     Parameters
     ----------
     arr : numpy.ndarray
@@ -448,6 +626,9 @@ def convert_color_space(arr, current, desired):
         The desired color space, should be a valid value for (0028,0004)
         *Photometric Interpretation*. One of ``'RGB'``, ``'YBR_FULL'``,
         ``'YBR_FULL_422'``.
+    per_frame : bool, optional
+        If ``True`` and the input array contains multiple frames then process
+        each frame individually to reduce memory usage. Default ``False``.
 
     Returns
     -------
@@ -463,7 +644,7 @@ def convert_color_space(arr, current, desired):
       <https://www.ijg.org/files/T-REC-T.871-201105-I!!PDF-E.pdf>`_),
       Section 7
     """
-    def _no_change(arr):
+    def _no_change(arr: "np.ndarray") -> "np.ndarray":
         return arr
 
     _converters = {
@@ -487,14 +668,19 @@ def convert_color_space(arr, current, desired):
         converter = _converters[current][desired]
     except KeyError:
         raise NotImplementedError(
-            "Conversion from {0} to {1} is not supported."
-            .format(current, desired)
+            f"Conversion from {current} to {desired} is not supported."
         )
 
+    if len(arr.shape) == 4 and per_frame:
+        for idx, frame in enumerate(arr):
+            arr[idx] = converter(frame)
+
+        return arr
+
     return converter(arr)
 
 
-def _convert_RGB_to_YBR_FULL(arr):
+def _convert_RGB_to_YBR_FULL(arr: "np.ndarray") -> "np.ndarray":
     """Return an ndarray converted from RGB to YBR_FULL color space.
 
     Parameters
@@ -522,21 +708,21 @@ def _convert_RGB_to_YBR_FULL(arr):
         [[+0.299, -0.299 / 1.772, +0.701 / 1.402],
          [+0.587, -0.587 / 1.772, -0.587 / 1.402],
          [+0.114, +0.886 / 1.772, -0.114 / 1.402]],
-        dtype=np.float
+        dtype=np.float32
     )
 
-    arr = np.dot(arr, rgb_to_ybr)
+    arr = np.matmul(arr, rgb_to_ybr, dtype=np.float32)
     arr += [0.5, 128.5, 128.5]
     # Round(x) -> floor of (arr + 0.5) : 0.5 added in previous step
-    arr = np.floor(arr)
+    np.floor(arr, out=arr)
     # Max(0, arr) -> 0 if 0 >= arr, arr otherwise
     # Min(arr, 255) -> arr if arr <= 255, 255 otherwise
-    arr = np.clip(arr, 0, 255)
+    np.clip(arr, 0, 255, out=arr)
 
     return arr.astype(orig_dtype)
 
 
-def _convert_YBR_FULL_to_RGB(arr):
+def _convert_YBR_FULL_to_RGB(arr: "np.ndarray") -> "np.ndarray":
     """Return an ndarray converted from YBR_FULL to RGB color space.
 
     Parameters
@@ -562,33 +748,36 @@ def _convert_YBR_FULL_to_RGB(arr):
         [[1.000, 1.000, 1.000],
          [0.000, -0.114 * 1.772 / 0.587, 1.772],
          [1.402, -0.299 * 1.402 / 0.587, 0.000]],
-        dtype=np.float
+        dtype=np.float32
     )
 
-    arr = arr.astype(np.float)
+    arr = arr.astype(np.float32)
     arr -= [0, 128, 128]
-    arr = np.dot(arr, ybr_to_rgb)
 
     # Round(x) -> floor of (arr + 0.5)
-    arr = np.floor(arr + 0.5)
+    np.matmul(arr, ybr_to_rgb, out=arr)
+    arr += 0.5
+    np.floor(arr, out=arr)
     # Max(0, arr) -> 0 if 0 >= arr, arr otherwise
     # Min(arr, 255) -> arr if arr <= 255, 255 otherwise
-    arr = np.clip(arr, 0, 255)
+    np.clip(arr, 0, 255, out=arr)
 
     return arr.astype(orig_dtype)
 
 
-def dtype_corrected_for_endianness(is_little_endian, numpy_dtype):
+def dtype_corrected_for_endianness(
+    is_little_endian: bool, numpy_dtype: "np.dtype"
+) -> "np.dtype":
     """Return a :class:`numpy.dtype` corrected for system and :class:`Dataset`
     endianness.
 
     Parameters
     ----------
     is_little_endian : bool
-        The endianess of the affected :class:`~pydicom.dataset.Dataset`.
+        The endianness of the affected :class:`~pydicom.dataset.Dataset`.
     numpy_dtype : numpy.dtype
         The numpy data type used for the *Pixel Data* without considering
-        endianess.
+        endianness.
 
     Raises
     ------
@@ -599,7 +788,7 @@ def dtype_corrected_for_endianness(is_little_endian, numpy_dtype):
     -------
     numpy.dtype
         The numpy data type used for the *Pixel Data* without considering
-        endianess.
+        endianness.
     """
     if is_little_endian is None:
         raise ValueError("Dataset attribute 'is_little_endian' "
@@ -611,7 +800,12 @@ def dtype_corrected_for_endianness(is_little_endian, numpy_dtype):
     return numpy_dtype
 
 
-def _expand_segmented_lut(data, fmt, nr_segments=None, last_value=None):
+def _expand_segmented_lut(
+    data: Tuple[int, ...],
+    fmt: str,
+    nr_segments: Optional[int] = None,
+    last_value: Optional[int] = None
+) -> List[int]:
     """Return a list containing the expanded lookup table data.
 
     Parameters
@@ -645,7 +839,7 @@ def _expand_segmented_lut(data, fmt, nr_segments=None, last_value=None):
     # Little endian: e.g. 0x0302 0x0100, big endian, e.g. 0x0203 0x0001
     indirect_ii = [3, 2, 1, 0] if '<' in fmt else [2, 3, 0, 1]
 
-    lut = []
+    lut: List[int] = []
     offset = 0
     segments_read = 0
     # Use `offset + 1` to account for possible trailing null
@@ -679,7 +873,7 @@ def _expand_segmented_lut(data, fmt, nr_segments=None, last_value=None):
                 lut.extend([y1] * length)
             else:
                 step = (y1 - y0) / length
-                vals = np.around(np.arange(y0 + step, y1 + step, step))
+                vals = np.around(np.linspace(y0 + step, y1, length))
                 lut.extend([int(vv) for vv in vals])
         elif opcode == 2:
             # C.7.9.2.3: Indirect segment
@@ -705,7 +899,7 @@ def _expand_segmented_lut(data, fmt, nr_segments=None, last_value=None):
         else:
             raise ValueError(
                 "Error expanding a segmented palette lookup table: "
-                "unknown segment type '{}'".format(opcode)
+                f"unknown segment type '{opcode}'"
             )
 
         segments_read += 1
@@ -715,7 +909,7 @@ def _expand_segmented_lut(data, fmt, nr_segments=None, last_value=None):
     return lut
 
 
-def get_expected_length(ds, unit='bytes'):
+def get_expected_length(ds: "Dataset", unit: str = 'bytes') -> int:
     """Return the expected length (in terms of bytes or pixels) of the *Pixel
     Data*.
 
@@ -758,14 +952,18 @@ def get_expected_length(ds, unit='bytes'):
         The expected length of the *Pixel Data* in either whole bytes or
         pixels, excluding the NULL trailing padding byte for odd length data.
     """
-    length = ds.Rows * ds.Columns * ds.SamplesPerPixel
-    length *= getattr(ds, 'NumberOfFrames', 1)
+    rows = cast(int, ds.Rows)
+    columns = cast(int, ds.Columns)
+    samples_per_pixel = cast(int, ds.SamplesPerPixel)
+    bits_allocated = cast(int, ds.BitsAllocated)
+
+    length = rows * columns * samples_per_pixel
+    length *= get_nr_frames(ds)
 
     if unit == 'pixels':
         return length
 
     # Correct for the number of bytes per pixel
-    bits_allocated = ds.BitsAllocated
     if bits_allocated == 1:
         # Determine the nearest whole number of bytes needed to contain
         #   1-bit pixel data. e.g. 10 x 10 1-bit pixels is 100 bits, which
@@ -781,7 +979,7 @@ def get_expected_length(ds, unit='bytes'):
     return length
 
 
-def get_image_pixel_ids(ds):
+def get_image_pixel_ids(ds: "Dataset") -> Dict[str, int]:
     """Return a dict of the pixel data affecting element's :func:`id` values.
 
     .. versionadded:: 1.4
@@ -838,7 +1036,70 @@ def get_image_pixel_ids(ds):
     return {kw: id(getattr(ds, kw, None)) for kw in keywords}
 
 
-def pixel_dtype(ds, as_float=False):
+def get_j2k_parameters(codestream: bytes) -> Dict[str, object]:
+    """Return a dict containing JPEG 2000 component parameters.
+
+    .. versionadded:: 2.1
+
+    Parameters
+    ----------
+    codestream : bytes
+        The JPEG 2000 (ISO/IEC 15444-1) codestream to be parsed.
+
+    Returns
+    -------
+    dict
+        A dict containing parameters for the first component sample in the
+        JPEG 2000 `codestream`, or an empty dict if unable to parse the data.
+        Available parameters are ``{"precision": int, "is_signed": bool}``.
+    """
+    try:
+        # First 2 bytes must be the SOC marker - if not then wrong format
+        if codestream[0:2] != b'\xff\x4f':
+            return {}
+
+        # SIZ is required to be the second marker - Figure A-3 in 15444-1
+        if codestream[2:4] != b'\xff\x51':
+            return {}
+
+        # See 15444-1 A.5.1 for format of the SIZ box and contents
+        ssiz = codestream[42]
+        if ssiz & 0x80:
+            return {"precision": (ssiz & 0x7F) + 1, "is_signed": True}
+
+        return {"precision": ssiz + 1, "is_signed": False}
+    except (IndexError, TypeError):
+        pass
+
+    return {}
+
+
+def get_nr_frames(ds: "Dataset") -> int:
+    """Return NumberOfFrames or 1 if NumberOfFrames is None.
+
+    Parameters
+    ----------
+    ds : dataset.Dataset
+        The :class:`~pydicom.dataset.Dataset` containing the Image Pixel module
+        corresponding to the data in `arr`.
+
+    Returns
+    -------
+    int
+        An integer for the NumberOfFrames or 1 if NumberOfFrames is None
+    """
+    nr_frames: Optional[int] = getattr(ds, 'NumberOfFrames', 1)
+    # 'NumberOfFrames' may exist in the DICOM file but have value equal to None
+    if nr_frames is None:
+        warnings.warn("A value of None for (0028,0008) 'Number of Frames' is "
+                      "non-conformant. It's recommended that this value be "
+                      "changed to 1")
+        nr_frames = 1
+
+    return nr_frames
+
+
+def pixel_dtype(ds: "Dataset", as_float: bool = False) -> "np.dtype":
     """Return a :class:`numpy.dtype` for the pixel data in `ds`.
 
     Suitable for use with IODs containing the Image Pixel module (with
@@ -892,7 +1153,7 @@ def pixel_dtype(ds, as_float=False):
         #   Data representation of the pixel samples
         #   0x0000 - unsigned int
         #   0x0001 - 2's complement (signed int)
-        pixel_repr = ds.PixelRepresentation
+        pixel_repr = cast(int, ds.PixelRepresentation)
         if pixel_repr == 0:
             dtype_str = 'uint'
         elif pixel_repr == 1:
@@ -900,8 +1161,8 @@ def pixel_dtype(ds, as_float=False):
         else:
             raise ValueError(
                 "Unable to determine the data type to use to contain the "
-                "Pixel Data as a value of '{}' for '(0028,0103) Pixel "
-                "Representation' is invalid".format(pixel_repr)
+                f"Pixel Data as a value of '{pixel_repr}' for '(0028,0103) "
+                "Pixel Representation' is invalid"
             )
     else:
         dtype_str = 'float'
@@ -910,7 +1171,7 @@ def pixel_dtype(ds, as_float=False):
     #   The number of bits allocated for each pixel sample
     #   PS3.5 8.1.1: Bits Allocated shall either be 1 or a multiple of 8
     #   For bit packed data we use uint8
-    bits_allocated = ds.BitsAllocated
+    bits_allocated = cast(int, ds.BitsAllocated)
     if bits_allocated == 1:
         dtype_str = 'uint8'
     elif bits_allocated > 0 and bits_allocated % 8 == 0:
@@ -918,8 +1179,8 @@ def pixel_dtype(ds, as_float=False):
     else:
         raise ValueError(
             "Unable to determine the data type to use to contain the "
-            "Pixel Data as a value of '{}' for '(0028,0100) Bits "
-            "Allocated' is invalid".format(bits_allocated)
+            f"Pixel Data as a value of '{bits_allocated}' for '(0028,0100) "
+            "Bits Allocated' is invalid"
         )
 
     # Check to see if the dtype is valid for numpy
@@ -927,8 +1188,8 @@ def pixel_dtype(ds, as_float=False):
         dtype = np.dtype(dtype_str)
     except TypeError:
         raise NotImplementedError(
-            "The data type '{}' needed to contain the Pixel Data is not "
-            "supported by numpy".format(dtype_str)
+            f"The data type '{dtype_str}' needed to contain the Pixel Data "
+            "is not supported by numpy"
         )
 
     # Correct for endianness of the system vs endianness of the dataset
@@ -939,7 +1200,7 @@ def pixel_dtype(ds, as_float=False):
     return dtype
 
 
-def reshape_pixel_array(ds, arr):
+def reshape_pixel_array(ds: "Dataset", arr: "np.ndarray") -> "np.ndarray":
     """Return a reshaped :class:`numpy.ndarray` `arr`.
 
     +------------------------------------------+-----------+----------+
@@ -978,9 +1239,9 @@ def reshape_pixel_array(ds, arr):
     |                        | Non-hierarchical,  |                       |
     |                        | SV1                |                       |
     +------------------------+--------------------+-----------------------+
-    | 1.2.840.10008.1.2.4.80 | JPEG-LS Lossless   | 1                     |
+    | 1.2.840.10008.1.2.4.80 | JPEG-LS Lossless   | 0                     |
     +------------------------+--------------------+-----------------------+
-    | 1.2.840.10008.1.2.4.81 | JPEG-LS Lossy      | 1                     |
+    | 1.2.840.10008.1.2.4.81 | JPEG-LS Lossy      | 0                     |
     +------------------------+--------------------+-----------------------+
     | 1.2.840.10008.1.2.4.90 | JPEG 2000 Lossless | 0                     |
     +------------------------+--------------------+-----------------------+
@@ -989,6 +1250,10 @@ def reshape_pixel_array(ds, arr):
     | 1.2.840.10008.1.2.5    | RLE Lossless       | 1                     |
     +------------------------+--------------------+-----------------------+
 
+    .. versionchanged:: 2.1
+
+        JPEG-LS transfer syntaxes changed to *Planar Configuration* of 0
+
     Parameters
     ----------
     ds : dataset.Dataset
@@ -1018,21 +1283,19 @@ def reshape_pixel_array(ds, arr):
     if not HAVE_NP:
         raise ImportError("Numpy is required to reshape the pixel array.")
 
-    nr_frames = getattr(ds, 'NumberOfFrames', 1)
-    nr_samples = ds.SamplesPerPixel
+    nr_frames = get_nr_frames(ds)
+    nr_samples = cast(int, ds.SamplesPerPixel)
 
     if nr_frames < 1:
         raise ValueError(
-            "Unable to reshape the pixel array as a value of {} for "
+            f"Unable to reshape the pixel array as a value of {nr_frames} for "
             "(0028,0008) 'Number of Frames' is invalid."
-            .format(nr_frames)
         )
 
     if nr_samples < 1:
         raise ValueError(
-            "Unable to reshape the pixel array as a value of {} for "
-            "(0028,0002) 'Samples per Pixel' is invalid."
-            .format(nr_samples)
+            f"Unable to reshape the pixel array as a value of {nr_samples} "
+            "for (0028,0002) 'Samples per Pixel' is invalid."
         )
 
     # Valid values for Planar Configuration are dependent on transfer syntax
@@ -1041,46 +1304,48 @@ def reshape_pixel_array(ds, arr):
         if transfer_syntax in ['1.2.840.10008.1.2.4.50',
                                '1.2.840.10008.1.2.4.57',
                                '1.2.840.10008.1.2.4.70',
+                               '1.2.840.10008.1.2.4.80',
+                               '1.2.840.10008.1.2.4.81',
                                '1.2.840.10008.1.2.4.90',
                                '1.2.840.10008.1.2.4.91']:
             planar_configuration = 0
-        elif transfer_syntax in ['1.2.840.10008.1.2.4.80',
-                                 '1.2.840.10008.1.2.4.81',
-                                 '1.2.840.10008.1.2.5']:
+        elif transfer_syntax in ['1.2.840.10008.1.2.5']:
             planar_configuration = 1
         else:
             planar_configuration = ds.PlanarConfiguration
 
         if planar_configuration not in [0, 1]:
             raise ValueError(
-                "Unable to reshape the pixel array as a value of {} for "
-                "(0028,0006) 'Planar Configuration' is invalid."
-                .format(planar_configuration)
+                "Unable to reshape the pixel array as a value of "
+                f"{planar_configuration} for (0028,0006) 'Planar "
+                "Configuration' is invalid."
             )
 
+    rows = cast(int, ds.Rows)
+    columns = cast(int, ds.Columns)
     if nr_frames > 1:
         # Multi-frame
         if nr_samples == 1:
             # Single plane
-            arr = arr.reshape(nr_frames, ds.Rows, ds.Columns)
+            arr = arr.reshape(nr_frames, rows, columns)
         else:
             # Multiple planes, usually 3
             if planar_configuration == 0:
-                arr = arr.reshape(nr_frames, ds.Rows, ds.Columns, nr_samples)
+                arr = arr.reshape(nr_frames, rows, columns, nr_samples)
             else:
-                arr = arr.reshape(nr_frames, nr_samples, ds.Rows, ds.Columns)
+                arr = arr.reshape(nr_frames, nr_samples, rows, columns)
                 arr = arr.transpose(0, 2, 3, 1)
     else:
         # Single frame
         if nr_samples == 1:
             # Single plane
-            arr = arr.reshape(ds.Rows, ds.Columns)
+            arr = arr.reshape(rows, columns)
         else:
             # Multiple planes, usually 3
             if planar_configuration == 0:
-                arr = arr.reshape(ds.Rows, ds.Columns, nr_samples)
+                arr = arr.reshape(rows, columns, nr_samples)
             else:
-                arr = arr.reshape(nr_samples, ds.Rows, ds.Columns)
+                arr = arr.reshape(nr_samples, rows, columns)
                 arr = arr.transpose(1, 2, 0)
 
     return arr
