@@ -3,23 +3,24 @@
 """Unit tests for the pydicom.dataelem module."""
 
 # Many tests of DataElement class are implied in test_dataset also
-
-import sys
+import math
 
 import pytest
 
 from pydicom import filewriter, config, dcmread
 from pydicom.charset import default_encoding
 from pydicom.data import get_testdata_file
+from pydicom.datadict import add_private_dict_entry
 from pydicom.dataelem import (
     DataElement,
     RawDataElement,
     DataElement_from_raw,
 )
 from pydicom.dataset import Dataset
+from pydicom.errors import BytesLengthException
 from pydicom.filebase import DicomBytesIO
 from pydicom.multival import MultiValue
-from pydicom.tag import Tag
+from pydicom.tag import Tag, BaseTag
 from pydicom.uid import UID
 from pydicom.valuerep import DSfloat
 
@@ -47,6 +48,33 @@ class TestDataElement:
         yield
         config.replace_un_with_known_vr = old_value
 
+    def test_AT(self):
+        """VR of AT takes Tag variants when set"""
+        elem1 = DataElement("OffendingElement", "AT", 0x100010)
+        elem2 = DataElement("OffendingElement", "AT", (0x10, 0x10))
+        elem3 = DataElement(
+            "FrameIncrementPointer", "AT", [0x540010, 0x540020]
+        )
+        elem4 = DataElement("OffendingElement", "AT", "PatientName")
+        assert isinstance(elem1.value, BaseTag)
+        assert isinstance(elem2.value, BaseTag)
+        assert elem1.value == elem2.value == elem4.value
+        assert elem1.value == 0x100010
+        assert isinstance(elem3.value, MultiValue)
+        assert len(elem3.value) == 2
+
+        # Test also using Dataset, and check 0x00000000 works
+        ds = Dataset()
+        ds.OffendingElement = 0
+        assert isinstance(ds.OffendingElement, BaseTag)
+        ds.OffendingElement = (0x0000, 0x0000)
+        assert isinstance(ds.OffendingElement, BaseTag)
+        assert ds.OffendingElement == 0
+
+        # An invalid Tag should throw an error
+        with pytest.raises(OverflowError):
+            _ = DataElement("OffendingElement", "AT", 0x100000000)
+
     def test_VM_1(self):
         """DataElement: return correct value multiplicity for VM > 1"""
         assert 3 == self.data_elementMulti.VM
@@ -75,6 +103,13 @@ class TestDataElement:
         assert isinstance(self.data_elementMulti.value[3], DSfloat)
         assert DSfloat('123.4') == self.data_elementMulti.value[3]
 
+    def test_DSFloat_conversion_auto_format(self):
+        """Test that strings are being auto-formatted correctly."""
+        data_element = DataElement((1, 2), "DS",
+                                   DSfloat(math.pi, auto_format=True))
+        assert math.pi == data_element.value
+        assert '3.14159265358979' == str(data_element.value)
+
     def test_backslash(self):
         """DataElement: String with '\\' sets multi-valued data_element."""
         data_element = DataElement((1, 2), "DS", r"42.1\42.2\42.3")
@@ -327,7 +362,7 @@ class TestDataElement:
 
         private_data_elem = ds[0x50f1100a]
         assert '[FNC Parameters]' == private_data_elem.name
-        assert 'UN' == private_data_elem.VR
+        assert 'SH' == private_data_elem.VR
 
     def test_private_repeater_tag(self):
         """Test that a known private tag in the repeater range is correctly
@@ -343,7 +378,7 @@ class TestDataElement:
 
         private_data_elem = ds[0x60211200]
         assert '[Overlay ID]' == private_data_elem.name
-        assert 'UN' == private_data_elem.VR
+        assert 'IS' == private_data_elem.VR
 
     def test_known_tags_with_UN_VR(self, replace_un_with_known_vr):
         """Known tags with VR UN are correctly decoded."""
@@ -578,3 +613,106 @@ class TestRawDataElement:
                              0, False, True)
         with pytest.raises(NotImplementedError):
             DataElement_from_raw(raw, default_encoding)
+
+    @pytest.fixture
+    def accept_wrong_length(self, request):
+        old_value = config.convert_wrong_length_to_UN
+        config.convert_wrong_length_to_UN = request.param
+        yield
+        config.convert_wrong_length_to_UN = old_value
+
+    @pytest.mark.parametrize("accept_wrong_length", [False], indirect=True)
+    def test_wrong_bytes_length_exception(self, accept_wrong_length):
+        """Check exception when number of raw bytes is not correct."""
+        raw = RawDataElement(Tag(0x00190000), 'FD', 1, b'1', 0, False, True)
+        with pytest.raises(BytesLengthException):
+            DataElement_from_raw(raw)
+
+    @pytest.mark.parametrize("accept_wrong_length", [True], indirect=True)
+    def test_wrong_bytes_length_convert_to_UN(self, accept_wrong_length):
+        """Check warning and behavior for incorrect number of raw bytes."""
+        value = b'1'
+        raw = RawDataElement(Tag(0x00190000), 'FD', 1, value, 0, False, True)
+        msg = (
+            r"Expected total bytes to be an even multiple of bytes per value. "
+            r"Instead received b'1' with length 1 and struct format 'd' which "
+            r"corresponds to bytes per value of 8. This occurred while trying "
+            r"to parse \(0019, 0000\) according to VR 'FD'. "
+            r"Setting VR to 'UN'."
+        )
+        with pytest.warns(UserWarning, match=msg):
+            raw_elem = DataElement_from_raw(raw)
+            assert 'UN' == raw_elem.VR
+            assert value == raw_elem.value
+
+    def test_read_known_private_tag_implicit(self):
+        fp = DicomBytesIO()
+        ds = Dataset()
+        ds.is_implicit_VR = True
+        ds.is_little_endian = True
+        ds[0x00410010] = RawDataElement(
+            Tag(0x00410010), "LO", 8, b"ACME 3.2", 0, True, True)
+        ds[0x00411001] = RawDataElement(
+            Tag(0x00411001), "US", 2, b"\x2A\x00", 0, True, True)
+        ds[0x00431001] = RawDataElement(
+            Tag(0x00431001), "SH", 8, b"Unknown ", 0, True, True)
+        ds.save_as(fp)
+        ds = dcmread(fp, force=True)
+        elem = ds[0x00411001]
+        assert elem.VR == "UN"
+        assert elem.name == "Private tag data"
+        assert elem.value == b"\x2A\x00"
+
+        add_private_dict_entry("ACME 3.2", 0x00410001, "US", "Some Number")
+        ds = dcmread(fp, force=True)
+        elem = ds[0x00411001]
+        assert elem.VR == "US"
+        assert elem.name == "[Some Number]"
+        assert elem.value == 42
+
+        # Unknown private tag is handled as before
+        elem = ds[0x00431001]
+        assert elem.VR == "UN"
+        assert elem.name == "Private tag data"
+        assert elem.value == b"Unknown "
+
+    def test_read_known_private_tag_explicit(self):
+        fp = DicomBytesIO()
+        ds = Dataset()
+        ds.is_implicit_VR = False
+        ds.is_little_endian = True
+        ds[0x00410010] = RawDataElement(
+            Tag(0x00410010), "LO", 8, b"ACME 3.2", 0, False, True)
+        ds[0x00411002] = RawDataElement(
+            Tag(0x00411002), "UN", 8, b"SOME_AET", 0, False, True)
+        ds.save_as(fp)
+        ds = dcmread(fp, force=True)
+        elem = ds[0x00411002]
+        assert elem.VR == "UN"
+        assert elem.name == "Private tag data"
+        assert elem.value == b"SOME_AET"
+
+        add_private_dict_entry("ACME 3.2", 0x00410002, "AE", "Some AET")
+        ds = dcmread(fp, force=True)
+        elem = ds[0x00411002]
+        assert elem.VR == "AE"
+        assert elem.name == "[Some AET]"
+        assert elem.value == "SOME_AET"
+
+    def test_read_known_private_tag_explicit_no_lookup(
+            self, dont_replace_un_with_known_vr):
+        add_private_dict_entry("ACME 3.2", 0x00410003, "IS", "Another Number")
+        fp = DicomBytesIO()
+        ds = Dataset()
+        ds.is_implicit_VR = False
+        ds.is_little_endian = True
+        ds[0x00410010] = RawDataElement(
+            Tag(0x00410010), "LO", 8, b"ACME 3.2", 0, False, True)
+        ds[0x00411003] = RawDataElement(
+            Tag(0x00411003), "UN", 8, b"12345678", 0, False, True)
+        ds.save_as(fp)
+        ds = dcmread(fp, force=True)
+        elem = ds[0x00411003]
+        assert elem.VR == "UN"
+        assert elem.name == "[Another Number]"
+        assert elem.value == b"12345678"
