@@ -142,8 +142,8 @@ class TestAT:
     def test_invalid_value_in_json(self):
         ds_json = ('{"00091001": {"vr": "AT", "Value": ["000910AG"]}, '
                    '"00091002": {"vr": "AT", "Value": ["00100010"]}}')
-        with pytest.warns(UserWarning, match='Invalid value "000910AG" for '
-                                             'AT element - ignoring it'):
+        with pytest.warns(UserWarning, match="Invalid value '000910AG' for "
+                                             "AT element - ignoring it"):
             ds = Dataset.from_json(ds_json)
             assert ds[0x00091001].value is None
             assert 0x00100010 == ds[0x00091002].value
@@ -160,7 +160,7 @@ class TestAT:
 
 class TestDataSetToJson:
     def test_json_from_dicom_file(self, no_numpy_use):
-        ds1 = Dataset(dcmread(get_testdata_file("CT_small.dcm")))
+        ds1 = dcmread(get_testdata_file("CT_small.dcm"))
         ds_json = ds1.to_json()
         ds2 = Dataset.from_json(ds_json)
         assert ds1 == ds2
@@ -270,15 +270,30 @@ class TestDataSetToJson:
         assert ds_json.index('"00100020"') < ds_json.index('"00100030"')
         assert ds_json.index('"00100030"') < ds_json.index('"00100040"')
 
+    def test_suppress_invalid_tags(self):
+        """Test tags that raise exceptions don't if suppress_invalid_tags True.
+        """
+        ds = Dataset()
+        ds.add_new(0x00100010, 'PN', ['Jane^Doe'])
+
+        with pytest.raises(Exception):
+            ds.to_json_dict()
+
+        ds_json = ds.to_json_dict(suppress_invalid_tags=True)
+
+        assert ds_json.get("00100010") is None
+
 
 class TestSequence:
     def test_nested_sequences(self):
         test1_json = get_testdata_file("test1.json")
         with open(test1_json) as f:
             with pytest.warns(UserWarning,
-                              match='no bulk data URI handler provided '):
+                              match='No bulk data URI handler provided '):
                 ds = Dataset.from_json(f.read())
             del ds.PixelData
+            assert ds.AcquisitionMatrix == [128, 0, 0, 32, None]
+
         ds2 = Dataset.from_json(ds.to_json())
         assert ds == ds2
 
@@ -298,8 +313,10 @@ class TestBinary:
         assert ds == ds1
 
     def test_invalid_inline_binary(self):
-        msg = ('"InlineBinary" of data element "00091002" '
-               'must be a bytes-like object.')
+        msg = (
+            "Invalid attribute value for data element '00091002' - the value "
+            "for 'InlineBinary' must be str, not int"
+        )
         ds_json = '{"00091002": {"vr": "OB", "InlineBinary": 42}}'
         with pytest.raises(TypeError, match=msg):
             Dataset.from_json(ds_json)
@@ -311,7 +328,7 @@ class TestBinary:
     def test_valid_bulkdata_uri(self):
         ds_json = ('{"00091002": {"vr": "OB", "BulkDataURI": '
                    '"http://example.com/bulkdatahandler"}}')
-        msg = r"no bulk data URI handler provided"
+        msg = r"No bulk data URI handler provided"
         with pytest.warns(UserWarning, match=msg):
             ds = Dataset.from_json(ds_json)
         assert 0x00091002 in ds
@@ -323,8 +340,10 @@ class TestBinary:
         assert 0x00091002 in ds
 
     def test_invalid_bulkdata_uri(self):
-        msg = ('"BulkDataURI" of data element "00091002" '
-               'must be a string.')
+        msg = (
+            "Invalid attribute value for data element '00091002' - the value "
+            "for 'BulkDataURI' must be str, not int"
+        )
         ds_json = '{"00091002": {"vr": "OB", "BulkDataURI": 42}}'
         with pytest.raises(TypeError, match=msg):
             Dataset.from_json(ds_json)
@@ -354,3 +373,77 @@ class TestBinary:
         ds = Dataset().from_json(json.dumps(json_data), bulk_data_reader)
 
         assert b'xyzzy' == ds[0x00091002].value
+
+    def test_bulk_data_reader_is_called_within_SQ(self):
+        def bulk_data_reader(_):
+            return b'xyzzy'
+
+        json_data = {
+            "003a0200": {
+                "vr": "SQ",
+                "Value": [
+                    {
+                        "54001010": {
+                            "vr": "OW",
+                            "BulkDataURI": "https://a.dummy.url"
+                        }
+                    }
+                ]
+            }
+        }
+
+        ds = Dataset().from_json(json.dumps(json_data), bulk_data_reader)
+
+        assert b'xyzzy' == ds[0x003a0200].value[0][0x54001010].value
+
+
+class TestNumeric:
+    def test_numeric_values(self):
+        ds = Dataset()
+
+        ds.add_new(0x0009100b, 'UL', 3000000000)
+        ds.add_new(0x0009100c, 'SL', -2000000000)
+        ds.add_new(0x0009100d, 'US', 40000)
+        ds.add_new(0x0009100e, 'SS', -22222)
+        ds.add_new(0x0009100f, 'FL', 3.14)
+        ds.add_new(0x00091010, 'FD', 3.14159265)
+        ds.add_new(0x00091014, 'IS', '42')
+        ds.add_new(0x00091015, 'DS', '3.14159265')
+        ds.add_new(0x00091102, 'US', 2)
+
+        ds_json = ds.to_json_dict()
+
+        assert ds_json['0009100B']['Value'] == [3000000000]
+        assert ds_json['0009100C']['Value'] == [-2000000000]
+        assert ds_json['0009100D']['Value'] == [40000]
+        assert ds_json['0009100E']['Value'] == [-22222]
+        assert ds_json['0009100F']['Value'] == [3.14]
+        assert ds_json['00091010']['Value'] == [3.14159265]
+        assert ds_json['00091014']['Value'] == [42]
+        assert ds_json['00091015']['Value'] == [3.14159265]
+        assert ds_json['00091102']['Value'] == [2]
+
+    def test_numeric_types(self):
+        ds = Dataset()
+
+        ds.add_new(0x0009100b, 'UL', 3000000000)
+        ds.add_new(0x0009100c, 'SL', -2000000000)
+        ds.add_new(0x0009100d, 'US', 40000)
+        ds.add_new(0x0009100e, 'SS', -22222)
+        ds.add_new(0x0009100f, 'FL', 3.14)
+        ds.add_new(0x00091010, 'FD', 3.14159265)
+        ds.add_new(0x00091014, 'IS', '42')
+        ds.add_new(0x00091015, 'DS', '3.14159265')
+        ds.add_new(0x00091102, 'US', 2)
+
+        ds_json = ds.to_json_dict()
+
+        assert type(ds_json['0009100B']['Value'][0]) == int
+        assert type(ds_json['0009100C']['Value'][0]) == int
+        assert type(ds_json['0009100D']['Value'][0]) == int
+        assert type(ds_json['0009100E']['Value'][0]) == int
+        assert type(ds_json['0009100F']['Value'][0]) == float
+        assert type(ds_json['00091010']['Value'][0]) == float
+        assert type(ds_json['00091014']['Value'][0]) == int
+        assert type(ds_json['00091015']['Value'][0]) == float
+        assert type(ds_json['00091102']['Value'][0]) == int
