@@ -26,23 +26,21 @@ There are the following possibilities:
 * PlanarConfiguration
 """
 
-from sys import byteorder
+from copy import deepcopy
 
 import pytest
 
-import pydicom
 from pydicom import config
-from pydicom.data import get_testdata_files
+from pydicom.data import get_testdata_file
 from pydicom.dataset import Dataset, FileMetaDataset
 from pydicom.filereader import dcmread
-
-from pydicom.tests._handler_common import ALL_TRANSFER_SYNTAXES
 from pydicom.pixel_data_handlers.util import convert_color_space
 from pydicom.uid import (
     ImplicitVRLittleEndian,
     ExplicitVRLittleEndian,
     DeflatedExplicitVRLittleEndian,
-    ExplicitVRBigEndian
+    ExplicitVRBigEndian,
+    AllTransferSyntaxes,
 )
 
 try:
@@ -58,7 +56,6 @@ try:
         get_pixeldata,
         unpack_bits,
         pack_bits,
-        pixel_dtype,
     )
 except ImportError:
     NP_HANDLER = None
@@ -69,11 +66,11 @@ except ImportError:
 # DEFL: Deflated Explicit VR Little Endian
 # EXPB: Explicit VR Big Endian
 # 1/1, 1 sample/pixel, 1 frame
-EXPL_1_1_1F = get_testdata_files("liver_1frame.dcm")[0]
-EXPB_1_1_1F = get_testdata_files("liver_expb_1frame.dcm")[0]
+EXPL_1_1_1F = get_testdata_file("liver_1frame.dcm")
+EXPB_1_1_1F = get_testdata_file("liver_expb_1frame.dcm")
 # 1/1, 1 sample/pixel, 3 frame
-EXPL_1_1_3F = get_testdata_files("liver.dcm")[0]
-EXPB_1_1_3F = get_testdata_files("liver_expb.dcm")[0]
+EXPL_1_1_3F = get_testdata_file("liver.dcm")
+EXPB_1_1_3F = get_testdata_file("liver_expb.dcm")
 # 1/1, 3 sample/pixel, 1 frame
 EXPL_1_3_1F = None
 EXPB_1_3_1F = None
@@ -81,73 +78,73 @@ EXPB_1_3_1F = None
 EXPL_1_3_XF = None
 EXPB_1_3_XF = None
 # 8/8, 1 sample/pixel, 1 frame
-DEFL_8_1_1F = get_testdata_files("image_dfl.dcm")[0]
-EXPL_8_1_1F = get_testdata_files("OBXXXX1A.dcm")[0]
-EXPB_8_1_1F = get_testdata_files("OBXXXX1A_expb.dcm")[0]
+DEFL_8_1_1F = get_testdata_file("image_dfl.dcm")
+EXPL_8_1_1F = get_testdata_file("OBXXXX1A.dcm")
+EXPB_8_1_1F = get_testdata_file("OBXXXX1A_expb.dcm")
 # 8/8, 1 sample/pixel, 2 frame
-EXPL_8_1_2F = get_testdata_files("OBXXXX1A_2frame.dcm")[0]
-EXPB_8_1_2F = get_testdata_files("OBXXXX1A_expb_2frame.dcm")[0]
+EXPL_8_1_2F = get_testdata_file("OBXXXX1A_2frame.dcm")
+EXPB_8_1_2F = get_testdata_file("OBXXXX1A_expb_2frame.dcm")
 # 8/8, 3 sample/pixel, 1 frame
-EXPL_8_3_1F = get_testdata_files("SC_rgb.dcm")[0]
-EXPB_8_3_1F = get_testdata_files("SC_rgb_expb.dcm")[0]
+EXPL_8_3_1F = get_testdata_file("SC_rgb.dcm")
+EXPB_8_3_1F = get_testdata_file("SC_rgb_expb.dcm")
 # 8/8, 3 samples/pixel, 1 frame, 3 x 3
-EXPL_8_3_1F_ODD = get_testdata_files('SC_rgb_small_odd.dcm')[0]
+EXPL_8_3_1F_ODD = get_testdata_file('SC_rgb_small_odd.dcm')
 # 8/8, 3 sample/pixel, 1 frame, YBR_FULL_422
-EXPL_8_3_1F_YBR422 = get_testdata_files('SC_ybr_full_422_uncompressed.dcm')[0]
+EXPL_8_3_1F_YBR422 = get_testdata_file('SC_ybr_full_422_uncompressed.dcm')
 # 8/8, 3 sample/pixel, 1 frame, YBR_FULL
-EXPL_8_3_1F_YBR = get_testdata_files('SC_ybr_full_uncompressed.dcm')[0]
+EXPL_8_3_1F_YBR = get_testdata_file('SC_ybr_full_uncompressed.dcm')
 # 8/8, 3 sample/pixel, 2 frame
-EXPL_8_3_2F = get_testdata_files("SC_rgb_2frame.dcm")[0]
-EXPB_8_3_2F = get_testdata_files("SC_rgb_expb_2frame.dcm")[0]
+EXPL_8_3_2F = get_testdata_file("SC_rgb_2frame.dcm")
+EXPB_8_3_2F = get_testdata_file("SC_rgb_expb_2frame.dcm")
 # 16/16, 1 sample/pixel, 1 frame
-IMPL_16_1_1F = get_testdata_files("MR_small_implicit.dcm")[0]
-EXPL_16_1_1F = get_testdata_files("MR_small.dcm")[0]
-EXPB_16_1_1F = get_testdata_files("MR_small_expb.dcm")[0]
+IMPL_16_1_1F = get_testdata_file("MR_small_implicit.dcm")
+EXPL_16_1_1F = get_testdata_file("MR_small.dcm")
+EXPB_16_1_1F = get_testdata_file("MR_small_expb.dcm")
 # Pixel Data with 128 bytes trailing padding
-EXPL_16_1_1F_PAD = get_testdata_files("MR_small_padded.dcm")[0]
+EXPL_16_1_1F_PAD = get_testdata_file("MR_small_padded.dcm")
 # 16/12, 1 sample/pixel, 10 frame
-EXPL_16_1_10F = get_testdata_files("emri_small.dcm")[0]
-EXPB_16_1_10F = get_testdata_files("emri_small_big_endian.dcm")[0]
+EXPL_16_1_10F = get_testdata_file("emri_small.dcm")
+EXPB_16_1_10F = get_testdata_file("emri_small_big_endian.dcm")
 # 16/16, 3 sample/pixel, 1 frame
-EXPL_16_3_1F = get_testdata_files("SC_rgb_16bit.dcm")[0]
-EXPB_16_3_1F = get_testdata_files("SC_rgb_expb_16bit.dcm")[0]
+EXPL_16_3_1F = get_testdata_file("SC_rgb_16bit.dcm")
+EXPB_16_3_1F = get_testdata_file("SC_rgb_expb_16bit.dcm")
 # 16/16, 3 sample/pixel, 2 frame
-EXPL_16_3_2F = get_testdata_files("SC_rgb_16bit_2frame.dcm")[0]
-EXPB_16_3_2F = get_testdata_files("SC_rgb_expb_16bit_2frame.dcm")[0]
+EXPL_16_3_2F = get_testdata_file("SC_rgb_16bit_2frame.dcm")
+EXPB_16_3_2F = get_testdata_file("SC_rgb_expb_16bit_2frame.dcm")
 # 32/32, 1 sample/pixel, 1 frame
-IMPL_32_1_1F = get_testdata_files("rtdose_1frame.dcm")[0]
-EXPB_32_1_1F = get_testdata_files("rtdose_expb_1frame.dcm")[0]
+IMPL_32_1_1F = get_testdata_file("rtdose_1frame.dcm")
+EXPB_32_1_1F = get_testdata_file("rtdose_expb_1frame.dcm")
 # 32/32, 1 sample/pixel, 15 frame
-IMPL_32_1_15F = get_testdata_files("rtdose.dcm")[0]
-EXPB_32_1_15F = get_testdata_files("rtdose_expb.dcm")[0]
+IMPL_32_1_15F = get_testdata_file("rtdose.dcm")
+EXPB_32_1_15F = get_testdata_file("rtdose_expb.dcm")
 # 32/32, 3 sample/pixel, 1 frame
-EXPL_32_3_1F = get_testdata_files("SC_rgb_32bit.dcm")[0]
-EXPB_32_3_1F = get_testdata_files("SC_rgb_expb_32bit.dcm")[0]
+EXPL_32_3_1F = get_testdata_file("SC_rgb_32bit.dcm")
+EXPB_32_3_1F = get_testdata_file("SC_rgb_expb_32bit.dcm")
 # 32/32, 3 sample/pixel, 2 frame
-EXPL_32_3_2F = get_testdata_files("SC_rgb_32bit_2frame.dcm")[0]
-EXPB_32_3_2F = get_testdata_files("SC_rgb_expb_32bit_2frame.dcm")[0]
+EXPL_32_3_2F = get_testdata_file("SC_rgb_32bit_2frame.dcm")
+EXPB_32_3_2F = get_testdata_file("SC_rgb_expb_32bit_2frame.dcm")
 
 # Transfer syntaxes supported by other handlers
 # JPEG Baseline (Process 1)
-JPEG_BASELINE_1 = get_testdata_files("SC_rgb_jpeg_dcmtk.dcm")[0]
+JPEG_BASELINE_1 = get_testdata_file("SC_rgb_jpeg_dcmtk.dcm")
 # JPEG Baseline (Process 2 and 4)
-JPEG_EXTENDED_2 = get_testdata_files("JPEG-lossy.dcm")[0]
+JPEG_EXTENDED_2 = get_testdata_file("JPEG-lossy.dcm")
 # JPEG Lossless (Process 14)
 JPEG_LOSSLESS_14 = None
 # JPEG Lossless (Process 14, Selection Value 1)
-JPEG_LOSSLESS_14_1 = get_testdata_files("SC_rgb_jpeg_gdcm.dcm")[0]
+JPEG_LOSSLESS_14_1 = get_testdata_file("SC_rgb_jpeg_gdcm.dcm")
 # JPEG-LS Lossless
-JPEG_LS_LOSSLESS = get_testdata_files("MR_small_jpeg_ls_lossless.dcm")[0]
+JPEG_LS_LOSSLESS = get_testdata_file("MR_small_jpeg_ls_lossless.dcm")
 # JPEG-LS Lossy
 JPEG_LS_LOSSY = None
 # JPEG2k Lossless
-JPEG_2K_LOSSLESS = get_testdata_files("emri_small_jpeg_2k_lossless.dcm")[0]
+JPEG_2K_LOSSLESS = get_testdata_file("emri_small_jpeg_2k_lossless.dcm")
 # JPEG2k
-JPEG_2K = get_testdata_files("JPEG2000.dcm")[0]
+JPEG_2K = get_testdata_file("JPEG2000.dcm")
 # RLE Lossless
-RLE = get_testdata_files("MR_small_RLE.dcm")[0]
+RLE = get_testdata_file("MR_small_RLE.dcm")
 # No Image Pixel module
-NO_PIXEL = get_testdata_files("rtplan.dcm")[0]
+NO_PIXEL = get_testdata_file("rtplan.dcm")
 
 
 # Transfer Syntaxes (non-retired + Explicit VR Big Endian)
@@ -158,7 +155,7 @@ SUPPORTED_SYNTAXES = [
     ExplicitVRBigEndian
 ]
 UNSUPPORTED_SYNTAXES = list(
-    set(ALL_TRANSFER_SYNTAXES) ^ set(SUPPORTED_SYNTAXES)
+    set(AllTransferSyntaxes) ^ set(SUPPORTED_SYNTAXES)
 )
 
 
@@ -191,12 +188,12 @@ class TestNoNumpy_NoNumpyHandler:
 
     def setup(self):
         """Setup the environment."""
-        self.original_handlers = pydicom.config.pixel_data_handlers
-        pydicom.config.pixel_data_handlers = []
+        self.original_handlers = config.pixel_data_handlers
+        config.pixel_data_handlers = []
 
     def teardown(self):
         """Restore the environment."""
-        pydicom.config.pixel_data_handlers = self.original_handlers
+        config.pixel_data_handlers = self.original_handlers
 
     def test_environment(self):
         """Check that the testing environment is as expected."""
@@ -235,7 +232,7 @@ class TestNoNumpy_NoNumpyHandler:
     def test_pixel_array_raises(self):
         """Test pixel_array raises exception for all syntaxes."""
         ds = dcmread(EXPL_16_1_1F)
-        for uid in ALL_TRANSFER_SYNTAXES:
+        for uid in AllTransferSyntaxes:
             ds.file_meta.TransferSyntaxUID = uid
             with pytest.raises(NotImplementedError,
                                match="UID of '{}'".format(uid)):
@@ -256,12 +253,12 @@ class TestNoNumpy_NumpyHandler:
 
     def setup(self):
         """Setup the environment."""
-        self.original_handlers = pydicom.config.pixel_data_handlers
-        pydicom.config.pixel_data_handlers = [NP_HANDLER]
+        self.original_handlers = config.pixel_data_handlers
+        config.pixel_data_handlers = [NP_HANDLER]
 
     def teardown(self):
         """Restore the environment."""
-        pydicom.config.pixel_data_handlers = self.original_handlers
+        config.pixel_data_handlers = self.original_handlers
 
     def test_environment(self):
         """Check that the testing environment is as expected."""
@@ -327,12 +324,12 @@ class TestNumpy_NoNumpyHandler:
 
     def setup(self):
         """Setup the environment."""
-        self.original_handlers = pydicom.config.pixel_data_handlers
-        pydicom.config.pixel_data_handlers = []
+        self.original_handlers = config.pixel_data_handlers
+        config.pixel_data_handlers = []
 
     def teardown(self):
         """Restore the environment."""
-        pydicom.config.pixel_data_handlers = self.original_handlers
+        config.pixel_data_handlers = self.original_handlers
 
     def test_environment(self):
         """Check that the testing environment is as expected."""
@@ -372,7 +369,7 @@ class TestNumpy_NoNumpyHandler:
     def test_pixel_array_raises(self):
         """Test pixel_array raises exception for all syntaxes."""
         ds = dcmread(EXPL_16_1_1F)
-        for uid in ALL_TRANSFER_SYNTAXES:
+        for uid in AllTransferSyntaxes:
             ds.file_meta.TransferSyntaxUID = uid
             with pytest.raises((NotImplementedError, RuntimeError)):
                 ds.pixel_array
@@ -425,12 +422,12 @@ class TestNumpy_NumpyHandler:
 
     def setup(self):
         """Setup the test datasets and the environment."""
-        self.original_handlers = pydicom.config.pixel_data_handlers
-        pydicom.config.pixel_data_handlers = [NP_HANDLER]
+        self.original_handlers = config.pixel_data_handlers
+        config.pixel_data_handlers = [NP_HANDLER]
 
     def teardown(self):
         """Restore the environment."""
-        pydicom.config.pixel_data_handlers = self.original_handlers
+        config.pixel_data_handlers = self.original_handlers
 
     def test_environment(self):
         """Check that the testing environment is as expected."""
@@ -1073,6 +1070,7 @@ class TestNumpy_NumpyHandler:
         ds.Rows = 10
         ds.Columns = 10
         ds.BitsAllocated = 16
+        ds.BitsStored = 16
         ds.PixelRepresentation = 0
         ds.SamplesPerPixel = 1
         ds.PhotometricInterpretation = 'MONOCHROME2'
@@ -1110,16 +1108,60 @@ class TestNumpy_GetPixelData:
         with pytest.raises(AttributeError, match=msg):
             get_pixeldata(ds)
 
-    def test_missing_required_elem(self):
+    def test_missing_required_elem_pixel_data_monochrome(self):
         """Tet get_pixeldata raises if dataset missing required element."""
-        ds = dcmread(EXPL_16_1_1F)
-        del ds.BitsAllocated
+        required_attrs = (
+            'BitsAllocated',
+            'BitsStored',
+            'Rows',
+            'Columns',
+            'SamplesPerPixel',
+            'PhotometricInterpretation',
+            'PixelRepresentation',
+        )
+        for attr in required_attrs:
+            ds = dcmread(EXPL_16_1_1F)
+            delattr(ds, attr)
+            msg = (
+                r"Unable to convert the pixel data as the following required "
+                r"elements are missing from the dataset: {}".format(attr)
+            )
+            with pytest.raises(AttributeError, match=msg):
+                get_pixeldata(ds)
+
+    def test_missing_required_elem_pixel_data_color(self):
+        """Tet get_pixeldata raises if dataset missing required element."""
+        ds = dcmread(EXPL_8_3_1F)
         del ds.Rows
-        del ds.SamplesPerPixel
+        del ds.Columns
         msg = (
             r"Unable to convert the pixel data as the following required "
-            r"elements are missing from the dataset: BitsAllocated, Rows, "
-            r"SamplesPerPixel"
+            r"elements are missing from the dataset: Rows, Columns"
+        )
+        with pytest.raises(AttributeError, match=msg):
+            get_pixeldata(ds)
+
+    def test_missing_conditionally_required_elem_pixel_data_color(self):
+        """Tet get_pixeldata raises if dataset missing required element."""
+        ds = dcmread(EXPL_8_3_1F)
+        del ds.PlanarConfiguration
+        msg = (
+            r"Unable to convert the pixel data as the following conditionally "
+            r"required element is missing from the dataset: "
+            r"PlanarConfiguration"
+        )
+        with pytest.raises(AttributeError, match=msg):
+            get_pixeldata(ds)
+
+    def test_missing_required_elem_float_pixel_data_monochrome(self):
+        """Tet get_pixeldata raises if dataset missing required element."""
+        ds = dcmread(IMPL_32_1_1F)
+        ds.FloatPixelData = ds.PixelData
+        del ds.PixelData
+        del ds.Rows
+        msg = (
+            r"Unable to convert the pixel data as the following required "
+            r"elements are missing from the dataset: Rows"
         )
         with pytest.raises(AttributeError, match=msg):
             get_pixeldata(ds)
@@ -1331,7 +1373,7 @@ class TestNumpy_PackBits:
     @pytest.mark.parametrize('output, input', REFERENCE_PACK_UNPACK)
     def test_pack(self, input, output):
         """Test packing data."""
-        assert output == pack_bits(np.asarray(input))
+        assert output == pack_bits(np.asarray(input), pad=False)
 
     def test_non_binary_input(self):
         """Test non-binary input raises exception."""
@@ -1339,20 +1381,33 @@ class TestNumpy_PackBits:
                            match=r"Only binary arrays \(containing ones or"):
             pack_bits(np.asarray([0, 0, 2, 0, 0, 0, 0, 0]))
 
-    def test_non_array_input(self):
-        """Test non 1D input raises exception."""
-        with pytest.raises(ValueError, match='Only 1D arrays are supported'):
-            pack_bits(
-                np.asarray(
-                    [[0, 0, 0, 0, 0, 0, 0, 0],
-                     [1, 0, 1, 0, 1, 0, 1, 0]]
-                )
-            )
+    def test_ndarray_input(self):
+        """Test non 1D input gets ravelled."""
+        arr = np.asarray(
+            [[0, 0, 0, 0, 0, 0, 0, 0],
+             [1, 0, 1, 0, 1, 0, 1, 0],
+             [1, 1, 1, 1, 1, 1, 1, 1]]
+        )
+        assert (3, 8) == arr.shape
+        b = pack_bits(arr, pad=False)
+        assert b'\x00\x55\xff' == b
+
+    def test_padding(self):
+        """Test odd length packed data is padded."""
+        arr = np.asarray(
+            [[0, 0, 0, 0, 0, 0, 0, 0],
+             [1, 0, 1, 0, 1, 0, 1, 0],
+             [1, 1, 1, 1, 1, 1, 1, 1]]
+        )
+        assert 3 == len(pack_bits(arr, pad=False))
+        b = pack_bits(arr, pad=True)
+        assert 4 == len(b)
+        assert 0 == b[-1]
 
     @pytest.mark.parametrize('output, input', REFERENCE_PACK_PARTIAL)
     def test_pack_partial(self, input, output):
         """Test packing data that isn't a full byte long."""
-        assert output == pack_bits(np.asarray(input))
+        assert output == pack_bits(np.asarray(input), pad=False)
 
     def test_functional(self):
         """Test against a real dataset."""
